unit Packform;
{$D-,L-}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, ComCtrls;

type
  TPackFormClass = class of TCustomPackForm;
  TCustomPackForm = class(TScrollBox)
  protected
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    procedure SetChildOrder(Child: TComponent; Order: Integer); override;
    property Align;
  public
    constructor Create(AOwner:TComponent); override;
    procedure Loaded; override;
    destructor Destroy; override;
  end;

  TPackForm = class(TCustomPackForm)
  private
    FOnCreate: TNotifyEvent;
    FOnDestroy: TNotifyEvent;
  public
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
    property Caption;
    property Align;
  published
    property DragCursor;
    property DragMode;
    property Enabled;
    property Color;
    property Ctl3D;
    property Font;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
    property OnCreate: TNotifyEvent read FOnCreate write FOnCreate;
    property OnDestroy: TNotifyEvent read FOnDestroy write FOnDestroy;
  end;
  TPackagePanel = class(TPanel)
  private
    FPackagePath : String;
    FErrorloadPackage,FALoadPackage,FBLoadPackage,
    FAUnloadPackage,FBUnloadPackage : TNotifyEvent;
  protected
  public
    Function LoadPack(Packagename : String) : Boolean;
    procedure UnLoadPack;
  published
    property PackagePath : String read FPackagePath write FPackagePath;
    property AfterLoadPackage : TNotifyEvent read FALoadPackage write FALoadPackage;
    property BeforeLoadPackage : TNotifyEvent read FBLoadPackage write FBLoadPackage;
    property OnErrorLoad : TNotifyEvent read FErrorLoadPackage write FErrorLoadPackage;
    property AfterUnLoadPackage : TNotifyEvent read FAUnLoadPackage write FAUnloadPackage;
    property BeforeUnLoadPackage : TNotifyEvent read FBUnloadPackage write FBUnloadPackage;
  end;

var
  PackageForm : TPackForm;
  PackInt : Variant;
  CurrentPacksPackage : TObject = nil;

procedure RegisterPackForm(Typename : TPersistentclass);
procedure UnRegisterPackForm(Typename : TPersistentclass);

implementation

resourcestring
  sResNotFound = 'Resource for PackForm is not found.';

Function TPackagePanel.LoadPack(Packagename : String) : Boolean;
begin
  if assigned(FBLoadPackage) then FBLoadPackage(self);
  try
    if CurrentPacksPackage<>nil then UnLoadPack;
    CurrentPacksPackage:=Pointer(LoadPackage(FPackagePath+'\'+Packagename));
    Result:=True;
  except
    Result:=False;
    if assigned(FErrorLoadPackage) then FErrorLoadPackage(self);
  end;
  if result then
    if assigned(FALoadPackage) then FALoadPackage(self);
end;

procedure TPackagePanel.UnLoadPack;
var Module : THandle;
    I : Integer;
    M : TMemoryBasicInformation;
begin
  if assigned(FBUnLoadPackage) then FBUnLoadPackage(self);
  if CurrentPacksPackage<>nil then
  begin
    Module:=THandle(CurrentPacksPackage);
    for i := Application.ComponentCount - 1 downto 0 do
    begin
      VirtualQuery(GetClass(Application.Components[i].ClassName), M, SizeOf(M));
      if (Module = 0) or (HMODULE(M.AllocationBase) = Module) then
      Application.Components[i].Free;
    end;
    UnRegisterModuleClasses(Module);
    UnLoadPackage(Module);
    CurrentPacksPackage:=nil;
  end;
  if assigned(FAUnLoadPackage) then FAUnLoadPackage(self);
end;

constructor TCustomPackForm.Create(AOwner:TComponent);
begin
  inherited Create(AOwner as TWincontrol);
  Parent:=AOwner as TWinControl;
  Align:=alclient;
end;

procedure TCustomPackForm.Loaded;
begin
  inherited Loaded;
end;

destructor TCustomPackForm.Destroy;
begin
  Parent:=nil;
  inherited destroy;
end;

procedure TCustomPackForm.GetChildren(Proc: TGetChildProc; Root: TComponent);
var
  I: Integer;
  OwnedComponent: TComponent;
begin
  inherited GetChildren(Proc, Root);
  if Root = Self then
  begin
    for I := 0 to ComponentCount - 1 do
    begin
      OwnedComponent := Components[I];
      if not OwnedComponent.HasParent then Proc(OwnedComponent);
    end;
  end;
end;

procedure TCustomPackForm.SetChildOrder(Child: TComponent; Order: Integer);
var
  I, J: Integer;
begin
  if Child is TControl then
    inherited SetChildOrder(Child, Order)
  else
  begin
    Dec(Order, ControlCount);
    J := -1;
    for I := 0 to ComponentCount - 1 do
      if not Components[I].HasParent then
      begin
        Inc(J);
        if J = Order then
        begin
          Child.ComponentIndex := I;
          Exit;
        end;
      end;
  end;
end;

constructor TPackForm.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  if (ClassType<>TPackForm) then
  begin
    if not InitInheritedComponent(Self, TPackForm) then
      raise EResNotFound.CreateFmt(sResNotFound, [ClassName]);
  end;
  try
    if Assigned(FOnCreate) then FOnCreate(Self);
  except
    Application.HandleException(Self);
  end;
end;

destructor TPackForm.Destroy;
begin
  if Assigned(FOnDestroy) then
  try
    FOnDestroy(Self);
  except
    Application.HandleException(Self);
  end;
  inherited Destroy;
end;

procedure RegisterPackForm(Typename : TPersistentclass);
begin
  Application.Processmessages;
  Registerclass(Typename);
  PackageForm:=(TComponentClass(Typename).Create(
    Application.Mainform.FindComponent('PackagePanel1')) as TPackForm);
//  PackInt:=PackageForm;
end;

procedure UnRegisterPackForm(Typename : TPersistentclass);
begin
  PackageForm.free;
  UnRegisterclass(Typename);
end;

end.


