unit PacksExp;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls,DsgnIntf, ExptIntf, ToolIntf, EditIntf, VirtIntf, TypInfo,Menus,
  PackForm;

type
  TPacksCustomModule = class(TCustomModule)
  public
    constructor Create(ARoot:IComponent); override;
    destructor Destroy; override;
    procedure ExecuteVerb(Index:integer); override;
    function getAttributes:TCustomModuleAttributes; override;
    function getVerb(Index:integer):string; override;
    function getVerbCount:integer; override;
    procedure Saving; override;
    procedure ValidateComponent(Component:IComponent); override;
  end;

  { TPacksExpert }

  TPacksExpert = class(TIExpert)
  public
    function GetName: string; override;
    function GetComment: string; override;
    function GetGlyph: HICON; override;
    function GetStyle: TExpertStyle; override;
    function GetState: TExpertState; override;
    function GetIDString: string; override;
    function GetAuthor: string; override;
    function GetPage: string; override;
    procedure Execute; override;
  end;

procedure Register;

implementation

const
  CRLF = #13#10;
  CRLF2 = #13#10#13#10;
  DefaultModuleFlags = [cmShowSource, cmShowForm, cmMarkModified, cmUnNamed];

resourcestring
  sPacksExpertAuthor = 'Alexander Mehlhorn';
  sPacksExpertName   = 'Form for Packs Packages';
  sPacksExpertDesc   = 'Creates a new Packs Package';

{ TPacksCustomModule }

constructor TPacksCustomModule.Create(ARoot:IComponent);
begin
  inherited Create(ARoot);
end;

destructor TPacksCustomModule.Destroy;
begin
  inherited Destroy;
end;

procedure TPacksCustomModule.ExecuteVerb(Index:integer);
begin
end;

function TPacksCustomModule.getAttributes:TCustomModuleAttributes;
begin
  Result:=[];
end;

function TPacksCustomModule.getVerb(Index:integer):string;
begin
end;

function TPacksCustomModule.getVerbCount:integer;
begin
  Result:=0;
end;

procedure TPacksCustomModule.Saving;
begin
end;

procedure TPacksCustomModule.ValidateComponent(Component:IComponent);
begin
end;

type
  TPacksCreator = class(TIModuleCreator)
  private
    FClass : string;
  public
    function Existing: Boolean; override;
    function GetAncestorName: string; override;
    function GetFileName: string; override;
    function GetFileSystem: string; override;
    function GetFormName: string; override;
    function NewModuleSource(const UnitIdent, FormIdent,
      AncestorIdent: string): string; override;
    procedure FormCreated(Form: TIFormInterface); override;
  end;

{ TPacksCreator }

function TPacksCreator.Existing:boolean;
begin
  Result:=False;
end;

function TPacksCreator.GetAncestorName:string;
begin
  Result:='PackForm';
end;

function TPacksCreator.GetFileName:string;
begin
  Result:='';
end;

function TPacksCreator.GetFileSystem:string;
begin
  Result:='';
end;

function TPacksCreator.GetFormName:string;
begin
  Result:=FClass;
end;

function TPacksCreator.NewModuleSource(const UnitIdent,FormIdent,AncestorIdent:string):string;
begin
  FClass:=AncestorIdent;
  Result:=       '{ This Unit was written for the Packs Solution Packages and'+CRLF;
  Result:=Result+'  can only be used as part thereof !                     '+CRLF;
  Result:=Result+'  WARNING ! Please use with care and proper calls        '+CRLF;
  Result:=Result+'  Packs Forms : written 1997 by Alexander Mehlhorn (AHM) }'+CRLF2;
  Result:=Result+'//Hints : Make sure you build using Run-time Packages'+CRLF2;
  Result:=Result+'unit '+UnitIdent+';'+CRLF2+
    'interface'+CRLF2+
    'uses'+CRLF+
    '  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,'+CRLF+
    '  ExtCtrls,PackForm;'+CRLF2+
    'type'+CRLF;
    Result:=Result+'  T'+FormIdent;
    Result:=Result+' = class(TPackForm)'+CRLF+
      '  private'+CRLF+
      '    { Private declarations }'+CRLF+
      '  public'+CRLF+
      '    { Public declarations }'+CRLF+
      '  end;'+CRLF2;
      Result:=Result+'var '+FormIdent+' : T'+FormIdent+';'+CRLF2;
    Result:=Result+
      'implementation'+CRLF2+
      '{$R *.DFM}'+CRLF2;
    Result:=Result+'{ Do not modify these Lines ever - AHM}'+CRLF;
    Result:=Result+
      'initialization'+CRLF+'  RegisterPackForm(T'+FormIdent+');'+CRLF;
    Result:=Result+
      'finalization'+CRLF+'  UnRegisterPackForm(T'+FormIdent+');'+CRLF;
    Result:=Result+
      'end.'+CRLF;
end;

procedure TPacksCreator.FormCreated(Form:TIFormInterface);
begin
  Form.Free;
end;

{ TPacksExpert }

procedure HandleException;
begin
  ToolServices.RaiseException(ReleaseException);
end;

procedure PacksExpert(ToolServices: TIToolServices);
var
  IModuleCreator : TPacksCreator;
  IModule : TIModuleInterface;
begin
  IModuleCreator:=TPacksCreator.Create;
  try
    IModule:=ToolServices.ModuleCreate(IModuleCreator,DefaultModuleFlags);
    IModule.Free;
  finally
    IModuleCreator.Free;
  end;
end;

function TPacksExpert.GetName: string;
begin
  try
    Result := sPacksExpertName;
  except
    HandleException;
  end;
end;

function TPacksExpert.GetComment: string;
begin
  try
    Result := sPacksExpertDesc;
  except
    HandleException;
  end;
end;

function TPacksExpert.GetGlyph: HICON;
begin
  Result:=0;
  try
    Result := LoadIcon(HInstance, 'NEWPacks');
  except
    HandleException;
  end;
end;

function TPacksExpert.GetStyle: TExpertStyle;
begin
  Result := esForm;
end;

function TPacksExpert.GetState: TExpertState;
begin
  try
    Result := [esEnabled];
  except
    HandleException;
  end;
end;

function TPacksExpert.GetIDString: string;
begin
  try
    Result := 'Borland.'+sPacksExpertName;
  except
    HandleException;
  end;
end;

function TPacksExpert.GetAuthor: string;
begin
  try
    Result := sPacksExpertAuthor;
  except
    HandleException;
  end;
end;

function TPacksExpert.GetPage: string;
begin
  try
    Result := 'New';
  except
    HandleException;
  end;
end;

procedure TPacksExpert.Execute;
begin
  try
    PacksExpert(ToolServices);
  except
    HandleException;
  end;
end;


procedure Register;
begin
  RegisterClass(TPackForm);
  RegisterComponents('Standard',[TPackagePanel]);
  RegisterCustomModule(TPackForm, TPacksCustomModule);
  RegisterLibraryExpert(TPacksExpert.Create);
end;

end.
